@@ -404,8 +404,8 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
     df = df.drop('timestamp', axis=1)
 
     if n_inputs==7 and past_points==40:
-        # X = df[['VO2_I', 'VCO2_I', 'VE_I', 'PetO2_I', 'PetCO2_I', 'VEVO2_I', 'VEVCO2_I']]
-        filter_vars = ['VO2_I', 'VCO2_I', 'VE_I', 'HR_I', 'RF_I', 'PetO2_I', 'PetCO2_I']
+        # filter_vars = ['VO2_I', 'VCO2_I', 'VE_I', 'HR_I', 'RF_I', 'PetO2_I', 'PetCO2_I']
+        filter_vars = ['VO2_I', 'VCO2_I', 'VE_I', 'PetO2_I', 'PetCO2_I', 'VEVO2_I', 'VEVCO2_I']
         X = df[filter_vars]
         XN = normalize(X)
         XN = XN.filter(filter_vars, axis=1)
@@ -440,9 +440,9 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
 
     tmp_df = pd.DataFrame()
     tmp_df['time'] = time
-    tmp_df['p_md'] = p_1
-    tmp_df['p_hv'] = p_2
-    tmp_df['p_sv'] = p_3
+    tmp_df['p_md'] = optimal_filter(np.asarray(time), np.asarray(p_1), 100)
+    tmp_df['p_hv'] = optimal_filter(np.asarray(time), np.asarray(p_2), 100)
+    tmp_df['p_sv'] = optimal_filter(np.asarray(time), np.asarray(p_3), 100)
 
     mod_col = tmp_df[['p_md', 'p_hv', 'p_sv']].iloc[:20].mean().idxmax()
     sev_col = tmp_df[['p_md', 'p_hv', 'p_sv']].iloc[-20:].mean().idxmax()
@@ -468,7 +468,7 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
     out_dict['VT1']['time'] = {}
     out_dict['VT2']['time'] = {}
 
-    VT1_index = int(out_df[(out_df['p_hv'] <= out_df['p_md']) & (out_df['p_hv'] > 0.1)].index[-1])
+    VT1_index = int(out_df[(out_df['p_hv'] <= out_df['p_md'])].index[-1])
     VT2_index = int(out_df[(out_df['p_hv'] <= out_df['p_sv']) & (out_df['p_hv'] > out_df['p_md'])].index[-1])
 
     out_dict['VT1']['time'] = df.iloc[VT1_index]['time']
@@ -499,16 +499,22 @@ def create_probabilities(duration=600, VT1=320, VT2=460):
     import numpy as np
 
     t = np.arange(1, duration + 1)
-    p_m = np.ones(np.shape(t))
-    p_m[t > VT1] = -1
-    p_h = - np.ones(np.shape(t))
-    p_h[((t > VT1) & (t < VT2))] = 1
-    p_s = -np.ones(np.shape(t))
-    p_s[t > VT2] = 1
 
-    p_mF = optimal_filter(t, p_m, 1000) + np.random.randn(len(t))/10
-    p_hF = optimal_filter(t, p_h, 600) + np.random.randn(len(t))/10
-    p_sF = optimal_filter(t, p_s, 600) + np.random.randn(len(t))/10
+    T_m = [0, VT1-1, VT1+1, VT2-1, VT2+1, duration]
+    T_h = [0, VT1-1, VT1+1, VT2-1, VT2+1, duration]
+    T_s = [0, VT1-1, VT1+1, VT2-1, VT2+1, duration]
+
+    p_m = [1, 1, -1, -1, -1, -1]
+    p_h = [-1, -1, 1, 1, -1, -1]
+    p_s = [-1, -1, -1, -1, 1, 1]
+
+    p_mF = optimal_filter(t, np.interp(t, T_m, p_m), 8000) + np.random.randn(len(t)) / 5
+    p_hF = optimal_filter(t, np.interp(t, T_h, p_h), 4000) + np.random.randn(len(t)) / 5
+    p_sF = optimal_filter(t, np.interp(t, T_s, p_s), 4000) + np.random.randn(len(t)) / 5
+
+    p_mF = np.interp(p_mF, (p_mF.min(), p_mF.max()), (-1, +1))
+    p_hF = np.interp(p_hF, (p_hF.min(), p_hF.max()), (-1, +1))
+    p_sF = np.interp(p_sF, (p_sF.min(), p_sF.max()), (-1, +1))
 
     return p_mF, p_hF, p_sF
 
@@ -537,7 +543,7 @@ def random_walk(length=1, scale_factor=1, variation=1):
 
     return [i/scale_factor for i in random_walk]
 
-def generate_CPET(generator, plot=False, fitness_group=None):
+def generate_CPET(generator, plot=False, fitness_group=None, VT1=None, VT2=None, duration=None):
     """Actually generates the CPET file
 
     Parameters:
@@ -570,9 +576,23 @@ def generate_CPET(generator, plot=False, fitness_group=None):
     else:
         db_df_sample = db_df[db_df['fitness_group'] == fitness_group].sample()
 
-    duration = int(db_df_sample.duration)
-    VT1 = int(db_df_sample.VT1)
-    VT2 = int(db_df_sample.VT2)
+    if duration == None:
+        duration = int(db_df_sample.duration)
+    if VT1 == None:
+        VT1 = int(db_df_sample.VT1)
+    if VT2 == None:
+        VT2 = int(db_df_sample.VT2)
+
+    # TODO: testing this opportunity
+    # if db_df_sample['fitness_group'].values[0] == 1:
+    #     VT1 = int(duration * 0.6)
+    #     VT2 = int(duration * 0.84)
+    # if db_df_sample['fitness_group'].values[0] == 2:
+    #     VT1 = int(duration * 0.49)
+    #     VT2 = int(duration * 0.75)
+    # if db_df_sample['fitness_group'].values[0] == 3:
+    #     VT1 = int(duration * 0.54)
+    #     VT2 = int(duration * 0.79)
 
     VO2_peak = int(db_df_sample.VO2peak)
     VCO2_peak = int(db_df_sample.VCO2peak)
@@ -632,20 +652,13 @@ def generate_CPET(generator, plot=False, fitness_group=None):
 
     noise_factor = random.randint(2, 4)/2
 
-    df['VO2_I'] = (np.asarray(VO2) - np.min(VO2))/(np.max((np.asarray(VO2) - np.min(VO2)))) * (VO2_peak - VO2_min) + VO2_min + \
-                  [random.uniform(-100, 100)*noise_factor for i in np.arange(duration)]
-    df['VCO2_I'] = (np.asarray(VCO2) - np.min(VCO2))/(np.max((np.asarray(VCO2) - np.min(VCO2)))) * (VCO2_peak - VCO2_min) + \
-                   VCO2_min + [random.uniform(-100, 100)*noise_factor for i in np.arange(duration)]
-    df['VE_I'] = (np.asarray(VE) - np.min(VE))/(np.max((np.asarray(VE) - np.min(VE)))) * (VE_peak - VE_min) + VE_min + \
-                 [random.uniform(-2, 2)*noise_factor for i in np.arange(duration)]
-    df['HR_I'] = (np.asarray(HR) - np.min(HR))/(np.max((np.asarray(HR) - np.min(HR)))) * (HR_peak - HR_min) + HR_min + \
-                 [random.uniform(-1, 1)*noise_factor for i in np.arange(duration)]
-    df['RF_I'] = (np.asarray(RF) - np.min(RF))/(np.max((np.asarray(RF) - np.min(RF)))) * (RF_peak - RF_min) + RF_min + \
-                 [random.uniform(-2, 2)*noise_factor for i in np.arange(duration)]
-    df['PetO2_I'] = (np.asarray(PetO2) - np.min(PetO2))/(np.max((np.asarray(PetO2) - np.min(PetO2)))) * (PetO2_peak - PetO2_min) + PetO2_min + \
-                    [random.uniform(-1, 1)*noise_factor for i in np.arange(duration)]
-    df['PetCO2_I'] = (np.asarray(PetCO2) - np.min(PetCO2))/(np.max((np.asarray(PetCO2) - np.min(PetCO2)))) * (PetCO2_peak - PetCO2_min) + PetCO2_min + \
-                     [random.uniform(-1, 1)*noise_factor for i in np.arange(duration)]
+    df['VO2_I'] = (np.asarray(VO2) - np.min(VO2))/(np.max((np.asarray(VO2) - np.min(VO2)))) * (VO2_peak - VO2_min) + VO2_min + [random.uniform(-100, 100)*noise_factor for i in np.arange(duration)]
+    df['VCO2_I'] = (np.asarray(VCO2) - np.min(VCO2))/(np.max((np.asarray(VCO2) - np.min(VCO2)))) * (VCO2_peak - VCO2_min) + VCO2_min + [random.uniform(-100, 100)*noise_factor for i in np.arange(duration)]
+    df['VE_I'] = (np.asarray(VE) - np.min(VE))/(np.max((np.asarray(VE) - np.min(VE)))) * (VE_peak - VE_min) + VE_min + [random.uniform(-2, 2)*noise_factor for i in np.arange(duration)]
+    df['HR_I'] = (np.asarray(HR) - np.min(HR))/(np.max((np.asarray(HR) - np.min(HR)))) * (HR_peak - HR_min) + HR_min + [random.uniform(-1, 1)*noise_factor*0.5 for i in np.arange(duration)]
+    df['RF_I'] = (np.asarray(RF) - np.min(RF))/(np.max((np.asarray(RF) - np.min(RF)))) * (RF_peak - RF_min) + RF_min + [random.uniform(-2, 2)*noise_factor for i in np.arange(duration)]
+    df['PetO2_I'] = (np.asarray(PetO2) - np.min(PetO2))/(np.max((np.asarray(PetO2) - np.min(PetO2)))) * (PetO2_peak - PetO2_min) + PetO2_min + [random.uniform(-1, 1)*noise_factor for i in np.arange(duration)]
+    df['PetCO2_I'] = (np.asarray(PetCO2) - np.min(PetCO2))/(np.max((np.asarray(PetCO2) - np.min(PetCO2)))) * (PetCO2_peak - PetCO2_min) + PetCO2_min + [random.uniform(-1, 1)*noise_factor for i in np.arange(duration)]
 
     df['VEVO2_I'] = df['VE_I']/df['VO2_I']
     df['VEVCO2_I'] = df['VE_I']/df['VCO2_I']
@@ -653,6 +666,12 @@ def generate_CPET(generator, plot=False, fitness_group=None):
     df.loc[df['time'] < VT1, 'domain'] = -1
     df.loc[df['time'] >= VT2, 'domain'] = 1
     df.loc[(df['time'] < VT2) & (df['time'] >= VT1), 'domain'] = 0
+    df['fitness_group'] = db_df_sample['fitness_group'].values[0]
+    df['Age'] = db_df_sample['Age'].values[0]
+    df['age_group'] = db_df_sample['age_group'].values[0]
+    df['gender'] = db_df_sample['gender'].values[0]
+    df['weight'] = db_df_sample['weight'].values[0]
+    df['height'] = db_df_sample['height'].values[0]
 
     if plot:
         terminal_plot([df['VO2_I'], df['VCO2_I']],
